# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Module for working with datatap."""

from .base_resource import AbstractResource, AbstractResourceController


class Datatap(AbstractResource):
    """Datatap Image item."""

    # All of the fields of Catalog objects as returned by the HPE Container
    # Platform API.
    # TODO: Verify this with the specification
    all_fields = (
        "id",
        "name",
        "description",
        "type",
        "status",
    )

    default_display_fields = [
        "id",
        "name",
        "description",
        "type",
        "status",
    ]

    # These fields are displayed by default, e.g. in tabulate()
    # TODO: Verify this with the specification
    # TODO: Pick a smaller subset, again based on the API response
    default_display_fields = all_fields

    @property
    def name(self):
        """@Field: from json['_embedded']['label']['name']."""
        try:
            return self.json["_embedded"]["label"]["name"]
        except KeyError:
            return ""

    @property
    def description(self):
        """@Field: from json['_embedded']['label']['description']."""
        try:
            return self.json["_embedded"]["label"]["description"]
        except KeyError:
            return ""

    @property
    def type(self):
        """@Field: from json['_embedded']['endpoint']['type']."""
        try:
            return self.json["_embedded"]["endpoint"]["type"]
        except KeyError:
            return ""

    @property
    def self_href(self):
        """@Field: from json['_links']['self']['href']."""
        try:
            return self.json["_links"]["self"]["href"]
        except KeyError:
            return ""

    @property
    def status(self):
        """@Field: from json['_embedded']['status']."""
        try:
            return self.json["_embedded"]["status"]
        except KeyError:
            return ""


class DatatapController(AbstractResourceController):
    """Class that users will interact with to work with datataps.

    An instance of this class is available in the
    `client.ContainerPlatformClient` with the attribute name
    :py:attr:`datatap <.client.ContainerPlatformClient.catalog>`. The methods
    of this class can be invoked using `client.datatap.method()`. See the
    example below:

    Examples
    --------
    >>> client = ContainerPlatformClient(...).create_session()
    >>> client.datatap.list()
    """

    base_resource_path = "/api/v1/dataconn"

    resource_list_path = "data_connectors"

    resource_class = Datatap

    def create_hdfs_with_kerberos(
        self,
        name,
        description,
        path_from_endpoint,
        kdc_data_host,
        kdc_data_port,
        realm,
        client_principal,
        browse_only,
        host,
        keytab,
        backup_host,
        type,
        port,
        read_only,
    ):
        """TODO.

        Parameters
        ----------
        name : [type]
            [description]
        description : [type]
            [description]
        path_from_endpoint : [type]
            [description]
        kdc_data_host : [type]
            [description]
        kdc_data_port : [type]
            [description]
        realm : [type]
            [description]
        client_principal : [type]
            [description]
        browse_only : [type]
            [description]
        host : [type]
            [description]
        keytab : [type]
            [description]
        backup_host : [type]
            [description]
        type : [type]
            [description]
        port : [type]
            [description]
        read_only : [type]
            [description]
        """
        _data = {
            "bdfs_root": {},
            "endpoint": {
                "kdc_data": [
                    {
                        "host": kdc_data_host,
                    }
                ],
                "realm": realm,
                "client_principal": client_principal,
                "browse_only": browse_only,
                "host": host,
                "keytab": keytab,
                "service_id": "hdfs",
                "backup_host": backup_host,
                "type": type,
                "port": str(port),
            },
            "flags": {"read_only": read_only},
            "label": {"name": name, "description": description},
        }

        if path_from_endpoint != "":
            _data["bdfs_root"]["path_from_endpoint"] = path_from_endpoint

        if kdc_data_port != "":
            _data["endpoint"]["kdc_data"]["kdc_data_port"] = kdc_data_port

        self.client._request(
            url=DatatapController.base_resource_path,
            http_method="post",
            description="datatap/create",
            data=_data,
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import json
import sys
import yaml

from hpecp.cli import base

if sys.version_info[0] >= 3:
    unicode = str


class LicenseProxy(object):
    """Proxy object to :py:attr:`<hpecp.client.license>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["delete", "delete_all", "list", "platform_id", "register"]

    @base.intercept_exception
    def platform_id(
        self,
    ):
        """Get the platform ID."""
        print(base.get_client().license.platform_id())

    def list(
        self,
        output="yaml",
        license_key_only=False,
    ):
        """Retrieve the list of licenses.

        :param output: how to display the output ['yaml'|'json']
        """
        response = base.get_client().license.list()
        if license_key_only:
            response = [
                str(unicode(li["LicenseKey"])) for li in response["Licenses"]
            ]
            print("\n".join(response))
        else:
            if output == "yaml":
                print(
                    yaml.dump(
                        yaml.load(
                            json.dumps(response),
                            Loader=yaml.FullLoader,
                        )
                    )
                )
            else:
                print(json.dumps(response))

    @base.intercept_exception
    def register(
        self,
        server_filename,
    ):
        """Register a license.

        :param server_filename: Filepath to the license on the server, e.g.
            '/srv/bluedata/license/LICENSE-1.txt'
        """
        print(
            base.get_client().license.register(server_filename=server_filename)
        )

    # TODO implement me!
    # def upload_with_ssh_key(
    #     self,
    #     server_filename,
    #     ssh_key_file=None,
    #     ssh_key_data=None,
    #     license_file=None,
    #     base64enc_license_data=None,
    # ):
    #     """Not implemented yet.

    #     Workaround:
    #     -----------
    #      - scp your license to '/srv/bluedata/license/' on the controller
    #      - run client.license.register(server_filename) to register
    #        the license
    #     """
    #     raise Exception(
    #         "Not implemented yet! Workaround: scp your license to"
    #         "'/srv/bluedata/license/'"
    #     )

    # TODO implement me!
    # def upload_with_ssh_pass(
    #     self,
    #     server_filename,
    #     ssh_username,
    #     ssh_password,
    #     license_file=None,
    #     base64enc_license_data=None,
    # ):
    #     """Not implemented yet.

    #     Workaround:
    #     -----------
    #      - scp your license to '/srv/bluedata/license/' on the controller
    #      - run client.license.register(server_filename) to register
    #        the license
    #     """
    #     raise Exception(
    #         "Not implemented yet! Workaround: scp your license to"
    #         "'/srv/bluedata/license/'"
    #     )

    @base.intercept_exception
    def delete(
        self,
        license_key,
    ):
        """Delete a license by LicenseKey.

        :param license_key: The license key, e.g. '1234 1234 ... 1234
            "SOMETEXT"'
        """
        base.get_client().license.delete(license_key=license_key)

    @base.intercept_exception
    def delete_all(
        self,
    ):
        """Delete all licenses."""
        response = base.get_client().license.list()
        all_license_keys = [
            str(unicode(li["LicenseKey"])) for li in response["Licenses"]
        ]
        for licence_key in all_license_keys:
            base.get_client().license.delete(license_key=licence_key)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""CLI Utilities."""

from collections import OrderedDict
from io import StringIO

import six
from six import string_types

# This class has been adapted from knack.
# See https://github.com/microsoft/knack/blob/master/LICENSE


class TextOutput(object):
    """Text output class."""

    @staticmethod
    def _dump_obj(data, stream):
        if isinstance(data, list):
            stream.write(str(len(data)))
        elif isinstance(data, dict):
            # We need to print something to avoid mismatching
            # number of columns if the value is None for some instances
            # and a dictionary value in other...
            stream.write("")
        else:
            to_write = data if isinstance(data, string_types) else str(data)
            stream.write(to_write)

    @staticmethod
    def _dump_row(data, stream):
        separator = ""
        if isinstance(data, (dict, list)):
            if isinstance(data, OrderedDict):
                values = data.values()
            elif isinstance(data, dict):
                values = [value for _, value in sorted(data.items())]
            else:
                values = data

            for value in values:
                stream.write(separator)
                TextOutput._dump_obj(value, stream)
                separator = "\t"
        elif isinstance(data, list):
            for value in data:
                stream.write(separator)
                TextOutput._dump_obj(value, stream)
                separator = "\t"
        elif isinstance(data, bool):
            TextOutput._dump_obj(str(data).lower(), stream)
        else:
            TextOutput._dump_obj(data, stream)
        stream.write("\n")

    @staticmethod
    def dump(data):
        """Dump the python object as text."""  # noqa: D202

        class MyStringIO(StringIO):
            def write(self, b):
                if six.PY2:
                    val = unicode(b)  # noqa: F821
                else:
                    val = b
                super(MyStringIO, self).write(val)

        io = MyStringIO()
        for item in data:
            if item is not None:
                TextOutput._dump_row(item, io)

        result = io.getvalue()
        io.close()
        return str(result)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import json
import jmespath
import sys
import yaml
from textwrap import dedent

from hpecp.cli import base
from hpecp.cli_utils import TextOutput


class ConfigProxy(object):
    """Proxy object to :py:attr:`<hpecp.client.config>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["examples", "get"]

    @base.intercept_exception
    def get(self, output="yaml", query=None):
        """Get the system config.

        :param output: how to display the output '[yaml|json|json-pp]'
        """
        if output not in ["yaml", "json", "json-pp", "text"]:
            print(
                (
                    "'output' parameter must be 'yaml', 'json', "
                    "'json-pp' or 'text'."
                ),
                file=sys.stderr,
            )
            sys.exit(1)

        if output == "yaml" and query is not None:
            print(
                "output=yaml is not supported with the query parameter.",
                file=sys.stderr,
            )
            sys.exit(1)

        response = base.get_client().config.get()

        if output == "yaml":
            print(
                yaml.dump(
                    yaml.load(
                        json.dumps(response),
                        Loader=yaml.FullLoader,
                    )
                )
            )
        else:
            if query is None:
                data = response
            else:
                data = jmespath.search(str(query), response)

            if output == "json-pp":
                print(json.dumps(data, indent=4, sort_keys=True))
            elif output == "text":
                print(TextOutput.dump(data))
            else:
                print(json.dumps(data))

    def examples(self):
        """Show examples for working with roles."""
        print(
            dedent(
                """\

                $ hpecp config get --query 'objects.[bds_global_version]' --output text
                5.1.1

                $ hpecp config get --query 'objects.[bds_global_version, bds_global_buildnumber]' --output text
                5.1.1
                2347

                """  # noqa:  E501
            )
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import sys
from hpecp.gateway import Gateway, GatewayStatus
from hpecp.cli import base


class GatewayProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.gateway>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "create_with_ssh_key",
            "delete",
            "get",
            "list",
            "states",
            "wait_for_state",
        ]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(GatewayProxy, self).new_instance("gateway", Gateway)

    @base.intercept_exception
    def create_with_ssh_key(
        self,
        ip,
        proxy_node_hostname,
        ssh_key=None,
        ssh_key_file=None,
        tags=[],
    ):
        """Create a Gateway using SSH key authentication.

        Parameters
        ----------
        ip : string
            The IP address of the proxy host.  Used for internal
            communication.
        proxy_node_hostname: string
            Clients will access cluster services will be accessed
            using this name.
        ssh_key: string
            The ssh key data as a string.  Alternatively, use the
            ssh_key_file parameter.
        ssh_key_file: string
            The file path to the ssh key.  Alternatively, use the
            ssh_key parameter.
        tags: string
            Tags to add to the gateway, for example:
            "{ 'tag1': 'foo', 'tag2', 'bar' }".
        """
        if ssh_key is None and ssh_key_file is None:
            print(
                "Either ssh_key or ssh_key_file must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if ssh_key is not None and ssh_key_file is not None:
            print(
                "Either ssh_key or ssh_key_file must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if ssh_key_file:
            with open(ssh_key_file) as f:
                ssh_key = f.read()

        gateway_id = base.get_client().gateway.create_with_ssh_key(
            ip=ip,
            proxy_node_hostname=proxy_node_hostname,
            ssh_key_data=ssh_key,
            tags=tags,
        )
        print(gateway_id)

    def states(
        self,
    ):
        """Return a list of valid states."""
        print([s.name for s in GatewayStatus])



#!/usr/bin/env python3

# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import collections
import configparser
import inspect
import os
import sys
from collections import OrderedDict
from textwrap import dedent

import fire
import six
from jinja2 import Environment

import hpecp
from hpecp import ContainerPlatformClient
from hpecp.cli.catalog import CatalogProxy
from hpecp.cli.config import ConfigProxy
from hpecp.cli.datatap import DatatapProxy
from hpecp.cli.gateway import GatewayProxy
from hpecp.cli.httpclient import HttpClientProxy
from hpecp.cli.k8scluster import K8sClusterProxy
from hpecp.cli.k8sworker import K8sWorkerProxy
from hpecp.cli.license import LicenseProxy
from hpecp.cli.lock import LockProxy
from hpecp.cli.role import RoleProxy
from hpecp.cli.tenant import TenantProxy
from hpecp.cli.user import UserProxy


def configure_cli():
    """Configure the CLI."""
    controller_api_host = None
    controller_api_port = None
    controller_use_ssl = None
    controller_verify_ssl = None
    controller_warn_ssl = None
    controller_username = None
    controller_password = None

    config_path = os.path.join(os.path.expanduser("~"), ".hpecp.conf")

    if os.path.exists(config_path):
        config_reader = ContainerPlatformClient.create_from_config_file()
        controller_api_host = config_reader.api_host
        controller_api_port = config_reader.api_port
        controller_use_ssl = config_reader.use_ssl
        controller_verify_ssl = config_reader.verify_ssl
        controller_warn_ssl = config_reader.warn_ssl
        controller_username = config_reader.username
        controller_password = config_reader.password

    sys.stdout.write("Controller API Host [{}]: ".format(controller_api_host))
    tmp = six.moves.input()
    if tmp != "":
        controller_api_host = tmp

    sys.stdout.write("Controller API Port [{}]: ".format(controller_api_port))
    tmp = six.moves.input()
    if tmp != "":
        controller_api_port = tmp

    sys.stdout.write(
        "Controller uses ssl (True|False) [{}]: ".format(controller_use_ssl)
    )
    tmp = six.moves.input()
    if tmp != "":
        controller_use_ssl = tmp

    sys.stdout.write(
        "Controller verify ssl (True|False) [{}]: ".format(
            controller_verify_ssl
        )
    )
    tmp = six.moves.input()
    if tmp != "":
        controller_verify_ssl = tmp

    sys.stdout.write(
        "Controller warn ssl (True|False) [{}]: ".format(controller_warn_ssl)
    )
    tmp = six.moves.input()
    if tmp != "":
        controller_warn_ssl = tmp

    sys.stdout.write("Controller Username [{}]: ".format(controller_username))
    tmp = six.moves.input()
    if tmp != "":
        controller_username = tmp

    sys.stdout.write("Controller Password [{}]: ".format(controller_password))
    tmp = six.moves.input()
    if tmp != "":
        controller_password = tmp

    config = configparser.ConfigParser()
    config["default"] = OrderedDict()
    config["default"]["api_host"] = controller_api_host
    config["default"]["api_port"] = str(controller_api_port)
    config["default"]["use_ssl"] = str(controller_use_ssl)
    config["default"]["verify_ssl"] = str(controller_verify_ssl)
    config["default"]["warn_ssl"] = str(controller_warn_ssl)
    config["default"]["username"] = controller_username
    config["default"]["password"] = controller_password

    with open(config_path, "w") as config_file:
        config.write(config_file)


class AutoComplete:
    """Shell autocompletion scripts.

    Example Usage:

    hpecp autocomplete bash > hpecp-bash.sh && source hpecp-bash.sh
    """

    def __dir__(self):
        """Return the CLI method names."""
        return ["bash"]

    def __init__(self, cli):
        """Create AutoCompletion class instance.

        Parameters
        ----------
        cli : CLI
            the owning cli instance
        """
        self.cli = cli

    def _get_metadata(self):

        modules = collections.OrderedDict()
        columns = collections.OrderedDict()

        for module_name in self.cli.__dict__.keys():

            # we manually define autocomplete for these methods
            if module_name in ["autocomplete", "configure_cli", "version"]:
                continue

            module = getattr(self.cli, module_name)
            function_names = dir(module)

            try:
                all_fields = getattr(module.resource_class, "all_fields")
            except Exception:
                all_fields = []

            function_parameters = collections.OrderedDict()

            # autcomplete should have most specific name first, e.g.
            # hpecp.tenant.create_xyz  before
            # hpecp.tenant.create
            for function_name in reversed(function_names):
                function = getattr(module, function_name)

                if six.PY2:
                    parameter_names = list(inspect.getargspec(function).args)
                else:
                    parameter_names = list(
                        inspect.getfullargspec(function).args
                    )

                # parameter_names = list(function.__code__.co_varnames)
                if "self" in parameter_names:
                    parameter_names.remove("self")

                # prefix parameter names with '--'
                parameter_names = list(map("--".__add__, parameter_names))

                function_parameters.update({function_name: parameter_names})

            modules[module_name] = function_parameters
            columns[module_name] = all_fields

            # _log = Logger.get_logger()
            # _log.debug(modules)
            # _log.debug(columns)

        return (modules, columns)

    def bash(self):
        """Create autocompletion script for bash."""
        __bash_template = dedent(
            """\
            _hpecp_complete()
                {
                local cur prev BASE_LEVEL

                COMPREPLY=()
                cur=${COMP_WORDS[COMP_CWORD]}
                prev=${COMP_WORDS[COMP_CWORD-1]}

                MODULE=${COMP_WORDS[1]}

                COMP_WORDS_AS_STRING=$(IFS=. ; echo "${COMP_WORDS[*]}")

                # if last input was > for redirecting to a file
                # perform file and directory autocompletion
                if echo "${prev}" | grep -q '>'
                then
                    _filedir;
                    return
                fi



                declare -A MODULE_COLUMNS=(
                    {% for module_name in modules %}
                        {% set column_names = " ".join(columns[module_name]) %}
                        ['{{module_name}}']="{{column_names}}"
                    {% endfor %}
                )

                {% raw %}
                # list has uniform behaviour as it is implemented in base.BaseProxy
                if [[ "${COMP_WORDS[2]}" == "list" ]];
                then

                    # if 'list' was the last word
                    if [[ "${prev}" == "list" ]];
                    then
                        COMPREPLY=( $(compgen -W "--columns --query" -- $cur) )
                        return
                    fi



                    # '--columns' was the last word and user is entering column names
                    if [[ "${COMP_WORDS[3]}" == "--columns"* && ${#COMP_WORDS[@]} -le 5 ]];
                    then
                        declare -a COLUMNS=(${MODULE_COLUMNS[$MODULE]})

                        local realcur prefix
                        realcur=${cur##*,} # everything after the last comma, e.g. a,b,c,d -> d
                        prefix=${cur%,*}   # everything before the lat comma, e.g. a,b,c,d -> a,b,c

                        if [[ "$cur" == *,* ]];
                        then
                            IFS=',' ENTERED_COLUMNS_LIST=($prefix)
                            unset IFS
                        else
                            IFS=',' ENTERED_COLUMNS_LIST=($prev)
                            unset IFS
                        fi

                        for COLUMN in ${COLUMNS[@]}; do
                            for ENTERED_COLUMN in ${ENTERED_COLUMNS_LIST[@]}; do
                                if [[ "${ENTERED_COLUMN}" == "${COLUMN}" ]]
                                then
                                    # remove columns already entered by user
                                    COLUMNS=(${COLUMNS[*]//$ENTERED_COLUMN/})
                                fi
                            done
                        done

                        if [[ "$cur" == *,* ]];
                        then
                            COMPREPLY=( $(compgen -W "${COLUMNS[*]}" -P "${prefix}," -S "," -- ${realcur}) )
                            compopt -o nospace
                            return
                        else
                            COMPREPLY=( $(compgen -W "${COLUMNS[*]}" -S "," -- ${realcur}) )
                            compopt -o nospace
                            return
                        fi
                    fi

                    # user has finished entering column list or query
                    if [[ ${#COMP_WORDS[@]} == 6 ]];
                    then
                        COMPREPLY=( $(compgen -W "--output" -- $cur) )
                        return
                    fi

                    if [[ "${COMP_WORDS[5]}" == "--output"*  ]];
                    then
                        if [[ "${COMP_WORDS[3]}" == "--columns"*  ]];
                        then
                            COMPREPLY=( $(compgen -W "table text" -- $cur) )
                            return
                        else
                            COMPREPLY=( $(compgen -W "json json-pp text" -- $cur) )
                            return
                        fi
                    fi

                    return
                fi
                {% endraw %}

                # if the last parameter was --*file perform
                # file and directory autocompletion
                if echo "${prev}" | grep -q '\-\-.*file$'
                then
                    _filedir;
                    return
                fi

                # if last input was > for redirecting to a file
                # perform file and directory autocompletion
                if echo "${prev}" | grep -q '>'
                then
                    _filedir;
                    return
                fi

                case "$COMP_WORDS_AS_STRING" in

                {% set module_names = " ".join(modules.keys()) %}
                {% for module_name in modules %}
                    {% set function_names = " ".join(modules[module_name].keys()).replace('_', '-') %}
                    {% for function_name in modules[module_name] %}
                        {% set param_names = " ".join(modules[module_name][function_name]).replace('_', '-') %}
                        {% if function_name == "list" %}
                            # do nothing - already handled above
                        {% else %}
                    *"hpecp.{{module_name}}.{{function_name.replace('_', '-')}}."*)
                        PARAM_NAMES="{{param_names}}"
                        for PARAM in ${PARAM_NAMES[@]}; do
                            PARAM="${PARAM//'\'}"
                            for WORD in ${COMP_WORDS[@]}; do
                                if [[ "${WORD}" == "${PARAM}" ]]
                                then
                                    # remove parameters already entered by user
                                    PARAM_NAMES=${PARAM_NAMES//$WORD/}
                                fi
                            done
                        done
                        COMPREPLY=( $(compgen -W "$PARAM_NAMES" -- $cur) )
                        ;;
                        {% endif %}
                    {% endfor %}
                    *"hpecp.{{module_name}}"*)
                        COMPREPLY=( $(compgen -W "{{function_names}}" -- $cur) )
                        ;;
                {% endfor %}
                    *"hpecp.autocomplete.bash"*)
                        COMPREPLY=( )
                        ;;
                    *"hpecp.autocomplete"*)
                        COMPREPLY=( $(compgen -W "bash" -- $cur) )
                        ;;
                    *"hpecp"*)
                        COMPREPLY=( $(compgen -W "autocomplete configure-cli version {{module_names}}" -- $cur) )
                        ;;
                esac
                return 0
            } &&
            complete -F _hpecp_complete hpecp
        """  # noqa: E501,W605
        )

        (modules, columns) = self._get_metadata()

        print(
            Environment(trim_blocks=True, lstrip_blocks=True)
            .from_string(__bash_template)
            .render(modules=modules, columns=columns),
            file=sys.stdout,
        )


def version(debug=False):
    """Display version information."""
    print("HPECP Version:   " + ContainerPlatformClient.version())

    if debug:
        print("HPECP Bin Path:  " + os.path.dirname(os.path.abspath(__file__)))
        print(
            "HPECP Lib Path:  "
            + os.path.dirname(os.path.abspath(hpecp.__file__))
        )
        print("Python Version:  " + sys.version.replace("\n", ""))
        print("Python Exe Path: " + sys.executable)
        print("Python Path:     " + os.pathsep.join(sys.path))
        print("System Path:     " + os.environ.get("PATH", ""))


class CLI(object):
    """Command Line Interface for the HPE Container Platform."""

    def __dir__(self):
        """Return modules names."""
        return list(vars(self))

    def __init__(self):
        """Create a CLI instance."""
        self.autocomplete = AutoComplete(self)
        self.configure_cli = configure_cli
        self.catalog = CatalogProxy()
        self.config = ConfigProxy()
        self.k8sworker = K8sWorkerProxy()
        self.k8scluster = K8sClusterProxy()
        self.tenant = TenantProxy()
        self.gateway = GatewayProxy()
        self.lock = LockProxy()
        self.license = LicenseProxy()
        self.httpclient = HttpClientProxy()
        self.user = UserProxy()
        self.role = RoleProxy()
        self.version = version
        self.datatap = DatatapProxy()


if __name__ == "__main__":
    fire.Fire(CLI)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import base64
import json
import six
import sys
import yaml

from hpecp.k8s_cluster import (
    K8sCluster,
    K8sClusterStatus,
    K8sClusterHostConfig,
)
from hpecp.cli import base


class K8sClusterProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.k8s_cluster>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "add_addons",
            "admin_kube_config",
            "create",
            "dashboard_url",
            "dashboard_token",
            "delete",
            "get",
            "get_available_addons",
            "get_installed_addons",
            "import_generic_cluster",
            "import_generic_cluster_with_json",
            "k8smanifest",
            "k8s_supported_versions",
            "list",
            "statuses",
            "upgrade_cluster",
            "wait_for_status",
        ]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(K8sClusterProxy, self).new_instance("k8s_cluster", K8sCluster)

    @base.intercept_exception
    def create(
        self,
        name,
        k8shosts_config,
        description=None,
        k8s_version=None,
        pod_network_range="10.192.0.0/12",
        service_network_range="10.96.0.0/12",
        pod_dns_domain="cluster.local",
        persistent_storage_local=False,
        persistent_storage_nimble_csi=False,
        addons=[],
    ):
        """Create a K8s Cluster.

        :param name: the cluster name
        :param k8shosts_config: k8s host ids and roles 'id1:master|worker,id2:
            master|worker,...'
        :param description: the cluster descripton
        :param k8s_version: e.g. 1.17.0
        :param pod_network_range: the pod network range,
            default='10.192.0.0/12'
        :param service_network_range: the service network range,
            default='10.96.0.0/12'
        :param pod_dns_domain: the pod dns domain, default='cluster.local'
        :param persistent_storage_local: True/False
        :param persistent_storage_nimble_csi: True/False
        :param addons: list of required addons. See:
            `hpecp k8scluster get-available-addons`
        """
        host_config = [
            K8sClusterHostConfig.create_from_list(h.split(":"))
            for h in k8shosts_config.split(",")
        ]

        print(
            base.get_client().k8s_cluster.create(
                name=name,
                description=description,
                k8s_version=k8s_version,
                pod_network_range=pod_network_range,
                service_network_range=service_network_range,
                pod_dns_domain=pod_dns_domain,
                persistent_storage_local=persistent_storage_local,
                persistent_storage_nimble_csi=persistent_storage_nimble_csi,
                k8shosts_config=host_config,
                addons=addons,
            )
        )

    def admin_kube_config(self, id):
        """Retrieve a K8s Cluster Admin Kube Config.

        :param id: the cluster ID
        """
        print(
            base.get_client()
            .k8s_cluster.get(id)
            .admin_kube_config.replace(
                "\\n",
                "\n",
            )
        )

    def dashboard_url(
        self,
        id,
    ):
        """Retrieve a K8s Cluster Dashboard URL.

        :param id: the cluster ID
        """
        url = (
            base.get_client().k8s_cluster.get(id=id).dashboard_endpoint_access
        )
        print(url)

    def dashboard_token(
        self,
        id,
    ):
        """Retrieve a K8s Cluster Dashboard Token.

        :param id: the cluster ID
        """
        token = base.get_client().k8s_cluster.get(id=id).dashboard_token
        if six.PY2:
            print(base64.b64decode(token.encode()))
        else:
            print(base64.b64decode(token.encode()).decode("utf-8"))

    @base.intercept_exception
    def k8smanifest(self):
        """Retrieve the k8smanifest."""
        response = base.get_client().k8s_cluster.k8smanifest()
        print(
            yaml.dump(
                yaml.load(
                    json.dumps(response),
                    Loader=yaml.FullLoader,
                )
            )
        )

    @base.intercept_exception
    def get_installed_addons(self, id):
        """Retrieve the installed addons on the cluster.

        :param id: get installed addons for a specific cluster
        """
        print(base.get_client().k8s_cluster.get(id=id).addons)

    @base.intercept_exception
    def get_available_addons(self, id=None, k8s_version=None):
        """Retrieve the available addons for a cluster.

        :param id: get available addons for a specific cluster (opt)
        :param k8s_version: get available addons for a cluster version (opt)
        """
        if id is not None and k8s_version is not None:
            print(
                "Either 'id' or 'k8s_version' parameter must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if id is None and k8s_version is None:
            print(
                "Either 'id' or 'k8s_version' parameter must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if id:
            print(base.get_client().k8s_cluster.get_available_addons(id=id))
        else:
            print(
                base.get_client().k8s_cluster.get_available_addons(
                    k8s_version=k8s_version
                )
            )

    @base.intercept_exception
    def add_addons(self, id, addons, wait_for_ready_sec=0):
        """Retrieve the installed addons on the cluster.

        :param id: get installed addons for a specific cluster
        :param addons: list of addons to install
        :param wait_for_ready_sec: wait for ready status
        (0 = do not wait)
        """
        if id is None:
            print("'id' parameter must be provided.", file=sys.stderr)
            sys.exit(1)
        if addons is None or not isinstance(addons, list) or len(addons) < 1:
            print(
                "'addons' must be a list with at least one entry.",
                file=sys.stderr,
            )
            sys.exit(1)

        base.get_client().k8s_cluster.add_addons(id=id, addons=addons)

        if wait_for_ready_sec > 0:
            self.wait_for_status(
                id=id, status=["ready"], timeout_secs=wait_for_ready_sec
            )

    def statuses(
        self,
    ):
        """Return a list of valid statuses."""
        print([s.name for s in K8sClusterStatus])

    @base.intercept_exception
    def k8s_supported_versions(
        self,
        output="json",
        major_filter=None,
        minor_filter=None,
        patch_filter=None,
    ):
        """Print a list of supported k8s versions.

        :param output: how to print the output, 'json' or 'text'
        :param major_filter: only return versions matching major_filter
        :param minor_filter: only return versions matching minor_filter
        :param patch_filter: only return versions matching patch_filter

        Example::

        hpecp k8scluster k8s_supported_versions --major-filter 1
            --minor-filter 17
        """
        if output not in [
            "json",
            "text",
        ]:
            print(
                "'output' parameter ust be 'json' or 'text'", file=sys.stderr
            )
            sys.exit(1)

        if major_filter is not None and not isinstance(major_filter, int):
            print("'major_filter' if provided must be an int", file=sys.stderr)
            sys.exit(1)

        if minor_filter is not None and not isinstance(minor_filter, int):
            print("'minor_filter' if provided must be an int", file=sys.stderr)
            sys.exit(1)

        if patch_filter is not None and not isinstance(patch_filter, int):
            print("'patch_filter' if provided must be an int", file=sys.stderr)
            sys.exit(1)

        if major_filter:
            major_filter = int(major_filter)

        if minor_filter:
            minor_filter = int(minor_filter)

        if patch_filter:
            patch_filter = int(patch_filter)

        vers = []
        for v in base.get_client().k8s_cluster.k8s_supported_versions():
            (major, minor, patch) = v.split(".")
            major = int(major)
            minor = int(minor)
            patch = int(patch)
            if (
                (major_filter is not None and major != major_filter)
                or (minor_filter is not None and minor != minor_filter)
                or (patch_filter is not None and patch != patch_filter)
            ):
                continue
            else:
                vers.append(v)

        if output == "json":
            print(vers)
        else:
            print(" ".join(vers))

    @base.intercept_exception
    def upgrade_cluster(
        self, id, k8s_upgrade_version, worker_upgrade_percent=20
    ):
        """Upgrade a cluster.

        TODO

        Returns
        -------
        TODO

        Raises
        ------
        APIException
        """
        base.get_client().k8s_cluster.upgrade_cluster(
            id, k8s_upgrade_version, worker_upgrade_percent
        )

    @base.intercept_exception
    def import_generic_cluster(
        self, name, description, pod_dns_domain, server_url, ca, bearer_token
    ):
        """Import a generic cluster.

        TODO

        Returns
        -------
        TODO

        Raises
        ------
        APIException
        """
        print(
            base.get_client().k8s_cluster.import_generic_cluster(
                name, description, pod_dns_domain, server_url, ca, bearer_token
            )
        )

    @base.intercept_exception
    def import_generic_cluster_with_json(
        self, json_file_path=None, json_content=None
    ):
        """Import a generic cluster from json.

        TODO

        Returns
        -------
        TODO

        Raises
        ------
        APIException
        """
        assert (
            json_file_path is not None or json_content is not None
        ), "Either --json-file-path or --json-content must be provided"

        assert (
            json_file_path is None or json_content is None
        ), "Either --json-file-path or --json-content must be provided."

        if json_file_path:
            try:
                with open(json_file_path, "r") as f:
                    json_content = f.read()
            except OSError:
                print(
                    "Could not open/read json-file-path: {}".format(
                        json_file_path
                    ),
                    file=sys.stderr,
                )
                sys.exit(1)

        json_content = json.loads(json_content)

        print(
            base.get_client().k8s_cluster.import_generic_cluster_with_json(
                json_content
            )
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import sys

from hpecp import APIException
from hpecp.exceptions import APIItemNotFoundException
from textwrap import dedent
from hpecp.catalog import Catalog
from hpecp.cli import base


class CatalogProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.catalog>`."""

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(CatalogProxy, self).new_instance("catalog", Catalog)

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "get",
            "list",
            "delete",
            "examples",
            "wait_for_state",
            "refresh",
            "install",
        ]

    def delete(self, id):
        """Not implemented."""
        raise AttributeError("'CatalogProxy' object has no attribute 'delete'")

    def refresh(self, catalog_id):
        """Refresh a catalog.

        Parameters
        ----------
        catalog_id : str
            The ID of the catalog - format: '/api/v1/catalog/[0-9]+'

        Examples
        --------
        > hpecp catalog refresh /api/v1/catalog/99

        """
        try:
            base.get_client().catalog.refresh(catalog_id)

            # TODO: Report progress of the refresh workflow
        except AssertionError as ae:
            print(ae, file=sys.stderr)
            sys.exit(1)
        except (APIException, APIItemNotFoundException) as e:
            print(e.message, file=sys.stderr)
            sys.exit(1)

    def install(self, catalog_id):
        """Install a catalog.

        Parameters
        ----------
        catalog_id : str
            The ID of the catalog - format: '/api/v1/catalog/[0-9]+'

        Examples
        --------
        > hpecp catalog install /api/v1/catalog/99

        """
        try:
            base.get_client().catalog.install(catalog_id)

            # TODO: Implement a way to check if the installation is actually
            # successful (and maybe report progress?) - wait_for_state()?
        except AssertionError as ae:
            print(ae, file=sys.stderr)
            sys.exit(1)
        except (APIException, APIItemNotFoundException) as e:
            print(e.message, file=sys.stderr)
            sys.exit(1)

    def examples(self):
        """Show examples for working with roles."""
        print(
            dedent(
                """\

                $  hpecp catalog list --query "[?state!='installed' && state!='installing'] | [*].[_links.self.href] | []"  --output json
                ["/api/v1/catalog/24", "/api/v1/catalog/27", "/api/v1/catalog/14", "/api/v1/catalog/11", "/api/v1/catalog/28", "/api/v1/catalog/18"]

                $  hpecp catalog list --query "[?state!='installed' && state!='installing'] | [*].[_links.self.href] | []"  --output text
                /api/v1/catalog/24
                /api/v1/catalog/27
                /api/v1/catalog/14
                /api/v1/catalog/11
                /api/v1/catalog/28
                /api/v1/catalog/18

                $  hpecp catalog list --query "[?state!='installed' && state!='installing'] | [*].[_links.self.href, distro_id]"  --output text
                /api/v1/catalog/29	bluedata/spark240juphub7xssl
                /api/v1/catalog/11	bluedata/ubuntu16
                /api/v1/catalog/21	bluedata/cdh632multi
                /api/v1/catalog/2	bluedata/spark231juphub7xssl
                """  # noqa:  E501
            )
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import

import re

from .base_resource import AbstractResource, AbstractResourceController

try:
    basestring
except NameError:
    basestring = str


class Role(AbstractResource):
    """Create an instance of Role from json data returned from the HPE
    Container Platform API.

    Users of this library are not expected to create an instance of this class.

    Parameters:
        json : str
            The json returned by the API representing a Gateway.

    Returns:
        Gateway:
            An instance of Gateway
    """

    all_fields = [
        "id",
        "name",
        "description",
    ]
    """All of the fields of Role objects as returned by the HPE Container
    Platform API"""

    default_display_fields = [
        "id",
        "name",
        "description",
    ]

    @property
    def id(self):
        """@Field: from json['_links']['self']['href'] - id format:
        '/api/v1/role/[0-9]+'"""
        return self.json["_links"]["self"]["href"]

    @property
    def name(self):
        """@Field: from json['label']['name']"""
        return self.json["label"]["name"]

    @property
    def description(self):
        """@Field: from json['label']['description']"""
        return self.json["label"]["description"]


class RoleController(AbstractResourceController):
    """This is the main class that users will interact with to work with roles.

    An instance of this class is available in the
    client.ContainerPlatformClient with the attribute name
    :py:attr:`role <.client.ContainerPlatformClient.role>`.  The methods of
    this class can be
    invoked using `client.role.method()`.  See the example below:

    Example::

        client = ContainerPlatformClient(...).create_session()
        client.role.get()

    """

    base_resource_path = "/api/v1/role/"
    resource_class = Role
    resource_list_path = "roles"



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import json
import sys

from hpecp.cli import base


class HttpClientProxy(object):
    """Proxy object to :py:attr:`<hpecp.client._request>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["delete", "get", "post", "put"]

    @base.intercept_exception
    def get(
        self,
        url,
    ):
        """Make HTTP GET request.

        Examples
        --------
        $ hpecp httpclient get /api/v1/workers
        """
        response = base.get_client()._request(
            url,
            http_method="get",
            description="CLI HTTP GET",
        )
        print(response.text, file=sys.stdout)

    @base.intercept_exception
    def delete(
        self,
        url,
    ):
        """Make HTTP DELETE request.

        Examples
        --------
        $ hpecp httpclient delete /api/v1/workers/1
        """
        base.get_client()._request(
            url,
            http_method="delete",
            description="CLI HTTP DELETE",
        )

    @base.intercept_exception
    def post(
        self,
        url,
        json_file="",
    ):
        """Make HTTP POST request.

        Examples
        --------
        $ cat > my.json <<-EOF
            {
                "external_identity_server":  {
                    "bind_pwd":"5ambaPwd@",
                    "user_attribute":"sAMAccountName",
                    "bind_type":"search_bind",
                    "bind_dn":"cn=Administrator,CN=Users,DC=samdom,DC=example,DC=com",
                    "host":"10.1.0.77",
                    "security_protocol":"ldaps",
                    "base_dn":"CN=Users,DC=samdom,DC=example,DC=com",
                    "verify_peer": false,
                    "type":"Active Directory",
                    "port":636
                }
            }
            EOF

            hpecp httpclient post /api/v2/config/auth --json-file my.json
        """
        with open(
            json_file,
            "r",
        ) as f:
            data = json.load(f)

        response = base.get_client()._request(
            url,
            http_method="post",
            data=data,
            description="CLI HTTP POST",
        )
        print(response.text, file=sys.stdout)

    @base.intercept_exception
    def put(
        self,
        url,
        json_file="",
    ):
        """Make HTTP PUT request.

        Examples
        --------
        $ hpecp httpclient put /api/v2/config/auth --json-file my.json
        """  # noqa: W293
        with open(
            json_file,
            "r",
        ) as f:
            data = json.load(f)

        response = base.get_client()._request(
            url,
            http_method="put",
            data=data,
            description="CLI HTTP PUT",
        )
        print(response.text, file=sys.stdout)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import

from enum import Enum

from requests.structures import CaseInsensitiveDict

from hpecp.base_resource import ResourceList

from .base_resource import AbstractResource, AbstractWaitableResourceController
from .exceptions import APIItemNotFoundException

try:
    basestring
except NameError:
    basestring = str


class GatewayStatus(Enum):
    """Bases: enum.Enum

    The statuses for a Gateway

    **Note:**

    The integer values do not have a meaning outside of this library.
    The API uses a string identifier with the status name rather than an
    integer value.
    """

    bundle = 1
    installing = 2
    installed = 3
    ready = 4
    unlicensed = 5
    configuring = 6
    configured = 7
    error = 8
    sysinfo = 9
    unconfiguring = 10
    deleting = 11
    storage_pending = 12
    storage_configuring = 13
    storage_error = 14
    decommission_in_progress = 15
    delete_in_progress = 16


class Gateway(AbstractResource):
    """Create an instance of Gateway from json data returned from the HPE

    Container Platform API. Users of this library are not expected to create an
    instance of this class.

    Parameters
    ----------
    json : str
        The json returned by the API representing a Gateway.

    Returns
    -------
    Gateway
        An instance of Gateway
    """

    # All of the fields of Gateway objects as returned by the HPE Container
    # Platform API
    all_fields = [
        "id",
        "hacapable",
        "propinfo",
        "approved_worker_pubkey",
        "schedule",
        "ip",
        "proxy_nodes_hostname",
        "hostname",
        "state",
        "status_info",
        "purpose",
        "sysinfo",
        "tags",
    ]

    # These fields are displayed by default, e.g. in tabulate()
    default_display_fields = [
        "id",
        "ip",
        "proxy_nodes_hostname",
        "hostname",
        "state",
        "status_info",
        "purpose",
        "tags",
    ]

    @property
    def state(self):
        """@Field: from json['state']"""
        return self.json["state"]

    @property
    def hacapable(self):
        """@Field: from json['hacapable']"""
        return self.json["hacapable"]

    @property
    def propinfo(self):
        """@Field: from json['propinfo']"""
        return self.json["propinfo"]

    @property
    def approved_worker_pubkey(self):
        """@Field: from json['approved_worker_pubkey']"""
        return self.json["approved_worker_pubkey"]

    @property
    def schedule(self):
        """@Field: from json['schedule']"""
        return self.json["schedule"]

    @property
    def ip(self):
        """@Field: from json['ip']"""
        return self.json["ip"]

    @property
    def proxy_nodes_hostname(self):
        """@Field: from json['proxy_nodes_hostname']"""
        try:
            return self.json["proxy_nodes_hostname"]
        except KeyError:
            return ""

    @property
    def hostname(self):
        """@Field: from json['hostname']"""
        return self.json["hostname"]

    @property
    def purpose(self):
        """@Field: from json['purpose']"""
        return self.json["purpose"]

    @property
    def status_info(self):
        """@Field: from json['status_info']"""
        return self.json["status_info"]

    @property
    def sysinfo(self):
        """@Field: from json['sysinfo']"""
        return self.json["sysinfo"]

    @property
    def tags(self):
        """@Field: from json['tags']"""
        return self.json["tags"]


class GatewayController(AbstractWaitableResourceController):
    """Class that users will interact with to work with Gateways. An instance
    of this class is available in the
    client.ContainerPlatformClient with the attribute name
    :py:attr:`gateway <.client.ContainerPlatformClient.gateway>`. The methods
    of this class can be invoked using `client.gateway.method()`. See the
    example below:

    Example
    -------
    >>> client = ContainerPlatformClient(...).create_session()
    >>> client.gateway.list()
    """

    base_resource_path = "/api/v1/workers"

    resource_list_path = "workers"

    resource_class = Gateway

    status_class = GatewayStatus

    status_fieldname = "state"

    # def create_with_ssh_password(self, username, password):
    #     """Not Implemented yet"""
    #     raise NotImplementedError()

    def create_with_ssh_key(
        self, ip, proxy_node_hostname, ssh_key_data, tags=[]
    ):
        """Create a gateway instance using SSH key credentials to access the
        host.

        Parameters
        ----------
        ip: str
            The IP address of the proxy host.  Used for internal
            communication.
        proxy_node_hostname: str
            Clients will access cluster services will be accessed using
            this name.
        ssh_key_data: str
            The ssh key data as a string.
        tags: list
            Tags to use, e.g. "{ 'tag1': 'foo', 'tag2', 'bar' }".

        Returns
        -------
        str
            gateway ID
        """

        assert isinstance(
            ip, basestring
        ), "'ip' must be provided and must be a string"
        assert isinstance(
            proxy_node_hostname, basestring
        ), "'proxy_node_hostname' must be provided and must be a string"
        assert isinstance(
            ssh_key_data, basestring
        ), "'ssh_key_data' must be provided and must be a string"

        data = {
            "ip": ip,
            "credentials": {
                "type": "ssh_key_access",
                "ssh_key_data": ssh_key_data,
            },
            "tags": tags,
            "proxy_nodes_hostname": proxy_node_hostname,
            "purpose": "proxy",
        }

        response = self.client._request(
            url="/api/v1/workers/",
            http_method="post",
            data=data,
            description="gateway/create_with_ssh_key",
        )
        return CaseInsensitiveDict(response.headers)["location"]

    def get(self, id):
        """Retrieve a Gateway by ID.

        Parameters
        ----------
        id: str
            The gateway ID - format: '/api/v1/workers/[0-9]+'

        Returns
        -------
        Gateway
            object representing a Gateway

        Raises
        ------
        APIException
        """
        worker = super(GatewayController, self).get(id)
        if worker.purpose != "proxy":
            raise APIItemNotFoundException(
                message="gateway not found with id: " + id,
                request_method="get",
                request_url=id,
            )
        return worker

    def list(self):
        """Make an API call to retrieve a list of Resources.

        Returns
        -------
        ResourceList
            The ResourceList will contain instances of the class defined by
            the property self.resource_class
        """
        resourceList = super(GatewayController, self).list()
        gateways = [gw for gw in resourceList.json if gw["purpose"] == "proxy"]
        return ResourceList(self.resource_class, gateways)

    # TODO refactor clients so implementation not required
    def wait_for_state(self, gateway_id, state=[], timeout_secs=1200):
        return super(GatewayController, self).wait_for_state(
            gateway_id, state, timeout_secs
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

from textwrap import dedent
from hpecp.user import User
from hpecp.cli import base


class UserProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.user>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["create", "get", "delete", "examples", "list"]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(UserProxy, self).new_instance("user", User)

    @base.intercept_exception
    def create(
        self,
        name,
        password,
        description,
        is_external=False,
    ):
        """Create a User.

        :param name: the user name
        :param password:  the password
        :param description: the user descripton

        """
        user_id = base.get_client().user.create(
            name=name,
            password=password,
            description=description,
            is_external=is_external,
        )
        print(user_id)

    def examples(self):
        """Show usage_examples of the list method."""
        print(
            dedent(
                """\

                hpecp user list --query '[?is_external]' --output json-pp
                """  # noqa: E501
            )
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import

import re

import polling
from requests.structures import CaseInsensitiveDict

try:
    basestring
except NameError:
    basestring = str


class LockController:
    def __init__(self, client):
        self.client = client

    def get(self):
        """Retrieve the locks"""
        return self.client._request(
            url="/api/v1/lock", http_method="get", description="lock/get_locks"
        ).json()

    def list(self):
        """Retrieve the locks"""
        return self.get()

    def create(self, reason=None):
        """Create a new lock.

        Arguments
        ---------
        reason: str
            Provide a reason for the lock.

        Raises
        ------
        APIException
        """
        data = {"reason": reason}
        response = self.client._request(
            url="/api/v1/lock",
            http_method="post",
            data=data,
            description="lock/set_lock",
        )
        return CaseInsensitiveDict(response.headers)["Location"]

    def delete(self, lock_id):
        """Delete a lock.

        Parameters
        ----------
        lock_id: str
            The lock id as retrieved with `get()`  Format:
            '/api/v1/lock/[0-9]+'

        Raises
        ------
        APIException
        """

        assert isinstance(
            lock_id, basestring
        ), "'lock_id' must be provided and must be a string"
        assert re.match(
            r"\/api\/v1\/lock\/[0-9]+", lock_id
        ), "'lock_id' must have format '/api/v1/lock/[0-9]+'"

        return self.client._request(
            url=lock_id, http_method="delete", description="lock/delete_lock"
        )

    def delete_all(self, timeout_secs=300):
        """Delete all locks.

        Parameters
        ----------
        timeout_secs: int
            How long to wait for internal locks (note these need to be
            cleared before external locks can be deleted)

        Raises
        ------
        APIException
        """

        try:
            polling.poll(
                lambda: len(self.get()["_embedded"]["internal_locks"]) == 0,
                step=10,
                poll_forever=False,
                timeout=timeout_secs,
            )
        except polling.TimeoutException:
            return False
        except Exception as e:
            self.client.log.error(e)
            return False

        if len(self.get()["_embedded"]["external_locks"]) > 0:
            for lock in self.get()["_embedded"]["external_locks"]:
                lock_id = lock["_links"]["self"]["href"]
                self.delete(lock_id)

        return True



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import


class ConfigController:
    def __init__(self, client):
        self.client = client

    def get(self):
        response = self.client._request(
            url="/api/v1/config",
            http_method="get",
            description="config/get",
        )
        return response.json()

    def auth(self, data):
        """
        Example::
            data = {
                "external_identity_server":  {
                    "bind_pwd":"5ambaPwd@",
                    "user_attribute":"sAMAccountName",
                    "bind_type":"search_bind",
                    "bind_dn":"cn=Administrator,CN=Users,DC=samdom,DC=example,DC=com",
                    "host":"10.1.0.77",
                    "security_protocol":"ldaps",
                    "base_dn":"CN=Users,DC=samdom,DC=example,DC=com",
                    "verify_peer": False,
                    "type":"Active Directory",
                    "port":636
                }
            }
        """
        self.client._request(
            url="/api/v2/config/auth",
            http_method="post",
            data=data,
            description="config/auth",
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from .base_resource import AbstractResource, AbstractResourceController


class Catalog(AbstractResource):
    """Catalog Image item."""

    # All of the fields of Catalog objects as returned by the HPE Container
    # Platform API.
    # TODO: Verify this with the specification
    all_fields = (
        "label_name",
        "label_description",
        "self_href",
        "feed",
        "distro_id",
        "version",
        "timestamp",
        "isdebug",
        "osclass",
        "logo_checksum",
        "logo_url",
        "documentation_checksum",
        "documentation_mimetype",
        "documentation_file",
        "state",
        "state_info",
    )

    default_display_fields = [
        "label_name",
        "label_description",
        "self_href",
    ]

    # These fields are displayed by default, e.g. in tabulate()
    # TODO: Verify this with the specification
    # TODO: Pick a smaller subset, again based on the API response
    default_display_fields = all_fields

    @property
    def label_name(self):
        """@Field: from json['label']['name']"""
        try:
            return self.json["label"]["name"]
        except KeyError:
            return ""

    @property
    def label_description(self):
        """@Field: from json['label']['description']"""
        try:
            return self.json["label"]["description"]
        except KeyError:
            return ""

    @property
    def self_href(self):
        """@Field: from json['_links']['self']['href']"""
        try:
            return self.json["_links"]["self"]["href"]
        except KeyError:
            return ""

    @property
    def feed(self):
        """@Field: from json['_links']['feed']"""
        try:
            return self.json["_links"]["feed"]
        except KeyError:
            return ""

    @property
    def distro_id(self):
        """@Field: from json['distro_id']"""
        try:
            return self.json["distro_id"]
        except KeyError:
            return ""

    @property
    def version(self):
        """@Field: from json['version']"""
        try:
            return self.json["version"]
        except KeyError:
            return ""

    @property
    def timestamp(self):
        """@Field: from json['timestamp']"""
        try:
            return self.json["timestamp"]
        except KeyError:
            return ""

    @property
    def isdebug(self):
        """@Field: from json['isdebug']"""
        try:
            return self.json["isdebug"]
        except KeyError:
            return ""

    @property
    def osclass(self):
        """@Field: from json['osclass']"""
        try:
            return self.json["osclass"]
        except KeyError:
            return ""

    @property
    def logo_checksum(self):
        """@Field: from json['logo']['checksum']"""
        try:
            return self.json["logo"]["checksum"]
        except KeyError:
            return ""

    @property
    def logo_url(self):
        """@Field: from json['logo']['url']"""
        try:
            return self.json["logo"]["url"]
        except KeyError:
            return ""

    @property
    def documentation_checksum(self):
        """@Field: from json['documentation']['checksum']"""
        try:
            return self.json["documentation"]["checksum"]
        except KeyError:
            return ""

    @property
    def documentation_mimetype(self):
        """@Field: from json['documentation']['mimetype']"""
        try:
            return self.json["documentation"]["mimetype"]
        except KeyError:
            return ""

    @property
    def documentation_file(self):
        """@Field: from json['documentation']['file']"""
        try:
            return self.json["documentation"]["file"]
        except KeyError:
            return ""

    @property
    def state(self):
        """@Field: from json['state']"""
        try:
            return self.json["state"]
        except KeyError:
            return ""

    @property
    def state_info(self):
        """@Field: from json['state_info']"""
        try:
            return self.json["state_info"]
        except KeyError:
            return ""


class CatalogController(AbstractResourceController):
    """Class that users will interact with to work with catalogs

    An instance of this class is available in the
    `client.ContainerPlatformClient` with the attribute name
    :py:attr:`catalog <.client.ContainerPlatformClient.catalog>`. The methods
    of this class can be invoked using `client.catalog.method()`. See the
    example below:

    Examples
    --------
    >>> client = ContainerPlatformClient(...).create_session()
    >>> client.catalog.list()
    """

    base_resource_path = "/api/v1/catalog"

    resource_list_path = "independent_catalog_entries"

    resource_class = Catalog

    def install(self, catalog_id):
        """Install the specified catalog.

        Parameters
        ----------
        catalog_id : str
            The ID of the catalog - format /api/v1/catalog/[0-9]+

        Raises
        ------
        APIItemNotFoundException
        APIItemConflictException
        APIException
        """
        # Make sure that the given catalog exists, other validations will also
        # be taken care of.
        self.get(catalog_id)

        _data = {"action": "install"}

        self.client._request(
            url=catalog_id,
            http_method="post",
            description="catalog/post/install",
            data=_data,
        )

    def refresh(self, catalog_id):
        """Refresh the specified catalog.

        Parameters
        ----------
        catalog_id : str
            The ID of the catalog - format /api/v1/catalog/[0-9]+

        Raises
        ------
        APIItemNotFoundException
        APIItemConflictException
        APIException
        """
        # Make sure that the given catalog exists, other validations will also
        # be taken care of.
        self.get(catalog_id)

        _data = {"action": "refresh"}

        self.client._request(
            url=catalog_id,
            http_method="post",
            description="catalog/post/refresh",
            data=_data,
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform Client."""

from __future__ import absolute_import

import ast
import codecs
import json
import os
import re
from configparser import SafeConfigParser

import pkg_resources
import requests
from requests.structures import CaseInsensitiveDict
from six import raise_from
from urllib3.exceptions import (
    ConnectTimeoutError,
    MaxRetryError,
    NewConnectionError,
)

from hpecp.exceptions import APIForbiddenException

from .catalog import CatalogController
from .config import ConfigController
from .datatap import DatatapController
from .exceptions import (
    APIException,
    APIItemConflictException,
    APIItemNotFoundException,
    APIUnknownException,
    ContainerPlatformClientException,
)
from .gateway import GatewayController
from .k8s_cluster import K8sClusterController
from .k8s_worker import K8sWorkerController
from .license import LicenseController
from .lock import LockController
from .logger import Logger
from .role import RoleController
from .tenant import TenantController
from .user import UserController

try:
    basestring
except NameError:
    basestring = str


class ContainerPlatformClient(object):
    """Client object for HPE Container Platform.

    This is the central object that users of this library work with.

    Parameters
    ----------
    username : str
        HPECP username
    password : str
        HPECP password
    api_host : str
        HPECP api_host
    api_port : int
        HPECP api_port
    use_ssl : bool:
        Connect to HPECP using SSL: True|False
    verify_ssl : bool|str
        See "https://requests.readthedocs.io/en/master/user/advanced/
        #ssl-cert-verification"
    warn_ssl : bool
        Disable ssl warnings
    tenant : str (optional)
        The tenant ID, e.g. /api/v1/tenant/2

    Returns
    -------
    ContainerPlatformClient:
        An instance of ContainerPlatformClient

    Notes
    -----
    Instantiating the ContainerPlatformClient does not make any connection
    to the HPE Container Platform API. The initial connection would be made
    by calling the method :py:meth:`create_session`.

    See Also
    --------
    :py:meth:`create_from_config_file` for an alternative way to create a
    ContainerPlatformClient instance
    :py:meth:`create_from_env` for an alternative way to create a
    ContainerPlatformClient instance
    """

    @classmethod
    def version(cls):
        """Retrieve the hpecp version information."""
        return pkg_resources.require("hpecp")[0].version

    @classmethod
    def create_from_config_file(
        cls, config_file="~/.hpecp.conf", profile=None
    ):
        """Create a ContainerPlatformClient object from a configuration file.

        Parameters
        ----------
        config_file : str
            The configuration filename and path
        profile : str
            If the configuration file has multiple profile sections, you
            can select the profile to use.

        Returns
        -------
        ContainerPlatformClient:
            An instance of ContainerPlatformClient is returned.

        Example
        -------
        Below is an example configuration file.

        [default]
        api_host = 127.0.0.1
        api_port = 8080
        use_ssl = True
        verify_ssl = False
        warn_ssl = False

        [demoserver]
        username = admin
        password = admin123
        tenant = /api/v1/tenant/2
        """
        _log = Logger.get_logger()

        if profile is None:
            profile = "default"

        if config_file.startswith("~"):
            file_path = config_file[1:]
            file_path = file_path.lstrip("/")
            config_file = os.path.join(os.path.expanduser("~"), file_path)

        if not os.path.exists(config_file):
            raise ContainerPlatformClientException(
                "Could not find configuration file '{}'".format(config_file)
            )

        config = SafeConfigParser()
        config.readfp(codecs.open(config_file, "r", "utf8"))

        assert (
            profile in config.sections()
        ), "'{}' section not found in '{}'".format(profile, config_file)
        assert (
            "username" in config[profile] or "username" in config["default"]
        ), (
            "'username' not found in section '{}' or in "
            "the default section".format(profile)
        )
        assert (
            "password" in config[profile] or "password" in config["default"]
        ), (
            "'password' not found in section '{}' "
            "or in the default section".format(profile)
        )
        assert (
            "api_host" in config[profile] or "api_host" in config["default"]
        ), (
            "'api_host' not found in section '{}' or in "
            "the default section".format(profile)
        )
        assert (
            "api_port" in config[profile] or "api_port" in config["default"]
        ), (
            "'api_port' not found in section '{}' or in "
            "the default section".format(profile)
        )
        assert (
            "use_ssl" in config[profile] or "use_ssl" in config["default"]
        ), (
            "'use_ssl' not found in section '{}' or in"
            "the default section".format(profile)
        )
        assert (
            "verify_ssl" in config[profile]
            or "verify_ssl" in config["default"]
        ), (
            "'verify_ssl' not found in section '{}' or in"
            "the default section".format(profile)
        )
        assert (
            "warn_ssl" in config[profile] or "warn_ssl" in config["default"]
        ), (
            "'warn_ssl' not found in section '{}' or in"
            "the default section".format(profile)
        )

        # tenant parameter is optional

        def get_config_value(key, profile):
            if key in config[profile]:
                _log.debug("Found '{}' in profile '{}'".format(key, profile))
                return config[profile][key]
            else:
                try:
                    val = config["default"][key]
                    _log.debug(
                        "Found '{}' in profile '{}'".format(key, "default")
                    )
                    return val
                except Exception:
                    _log.debug(
                        "Could not find '{}' in profile '{}'".format(
                            key, profile
                        )
                    )
                    return None

        username = str(get_config_value("username", profile))
        password = str(get_config_value("password", profile))
        api_host = str(get_config_value("api_host", profile))
        api_port = int(get_config_value("api_port", profile))
        use_ssl = str(get_config_value("use_ssl", profile))
        verify_ssl = str(get_config_value("verify_ssl", profile))
        warn_ssl = str(get_config_value("warn_ssl", profile))

        # optional parameter
        tenant = get_config_value("tenant", profile)
        if tenant:
            assert isinstance(tenant, str) and re.match(
                r"\/api\/v1\/tenant\/[0-9]+", tenant
            ), (
                "'tenant' must have format '/api/v1/tenant/[0-9]+' in '{}'"
            ).format(
                config_file
            )

        if use_ssl == "False":
            use_ssl = False
        else:
            use_ssl = True

        # verify_ssl could be a path
        if verify_ssl == "False":
            verify_ssl = False

        if warn_ssl == "False":
            warn_ssl = False
        else:
            warn_ssl = True

        return cls(
            username,
            password,
            api_host,
            api_port,
            use_ssl,
            verify_ssl,
            warn_ssl,
            tenant,
        )

    @classmethod
    def create_from_env(cls):
        """Create an instance of ContainerPlatformClient from environment variables.

        Variables
        ---------
        HPECP_USERNAME
        HPECP_PASSWORD
        HPECP_API_HOST
        HPECP_API_PORT
        HPECP_USE_SSL
        HPECP_VERIFY_SSL
        HPECP_WARN_SSL
        HPECP_TENANT

        See Also
        --------
        See ContainerPlatformClient
        :py:class:`constructor <ContainerPlatformClient>` for the paramaeter
        definitions.
        """
        try:
            HPECP_USERNAME = os.environ["HPECP_USERNAME"]
            HPECP_PASSWORD = os.environ["HPECP_PASSWORD"]
            HPECP_API_HOST = os.environ["HPECP_API_HOST"]
            HPECP_API_PORT = int(os.environ["HPECP_API_PORT"])

            HPECP_USE_SSL = ast.literal_eval(os.environ["HPECP_USE_SSL"])
            HPECP_VERIFY_SSL = ast.literal_eval(os.environ["HPECP_VERIFY_SSL"])
            HPECP_WARN_SSL = ast.literal_eval(os.environ["HPECP_WARN_SSL"])

            # Optional parameter
            HPECP_TENANT = os.getenv("HPECP_TENANT", default=None)

            if HPECP_TENANT:
                assert isinstance(HPECP_TENANT, str) and re.match(
                    r"\/api\/v1\/tenant\/[0-9]+", HPECP_TENANT
                ), (
                    "'tenant' must have format '/api/v1/tenant/[0-9]+' in "
                    "env var 'HPECP_TENANT'"
                )

        except KeyError as ke:
            raise ContainerPlatformClientException(
                "Required env var '{}' not found.".format(ke.args[0])
            )
        except ValueError as ve:
            # TODO replace with asssertions
            raise ContainerPlatformClientException(ve.args[0])

        return cls(
            username=HPECP_USERNAME,
            password=HPECP_PASSWORD,
            api_host=HPECP_API_HOST,
            api_port=HPECP_API_PORT,
            use_ssl=HPECP_USE_SSL,
            verify_ssl=HPECP_VERIFY_SSL,
            warn_ssl=HPECP_WARN_SSL,
            tenant=HPECP_TENANT,
        )

    def __init__(
        self,
        username=None,
        password=None,
        api_host=None,
        api_port=8080,
        use_ssl=True,
        verify_ssl=True,
        warn_ssl=False,
        tenant=None,
    ):
        """Create a Client object for interacting with HPE Container Platform.

        Parameters
        ----------
        username : str
            HPECP Username
        password : str
            HPECP Password
        api_host : str,
            HPECP API Host
        api_port : int, optional
            HPECP API Port, by default 8080
        use_ssl : bool, optional
            Connect to HPECP using SSL:, by default True
        verify_ssl : bool, optional
            See "https://requests.readthedocs.io/en/master/user/advanced/
            #ssl-cert-verification", by default True
        warn_ssl : bool, optional
            Disable ssl warnings, by default False
        tenant : str, optional
            The tenant ID, e.g. /api/v1/tenant/2
        """
        self._log = Logger.get_logger()

        if verify_ssl == "True":
            verify_ssl = True

        if verify_ssl == "False":
            verify_ssl = False

        self._log.debug(
            "ContainerPlatformClient() created with '{}'".format(
                {
                    "username": username,
                    "password": "********",
                    "api_host": api_host,
                    "api_port": api_port,
                    "use_ssl": use_ssl,
                    "verify_ssl": verify_ssl,
                    "warn_ssl": warn_ssl,
                    "tenant": tenant,
                }
            )
        )

        assert isinstance(
            username, basestring
        ), "'username' parameter must be of type string"
        assert isinstance(
            password, basestring
        ), "'password' parameter must be of type string"
        assert isinstance(
            api_host, basestring
        ), "'api_host' parameter must be of type string"
        assert isinstance(
            api_port, int
        ), "'api_port' parameter must be of type int"
        assert isinstance(
            use_ssl, bool
        ), "'use_ssl' parameter must be of type bool"
        assert isinstance(verify_ssl, bool) or (
            isinstance(verify_ssl, basestring)
            and os.access(verify_ssl, os.R_OK)
        ), (
            "'verify_ssl' parameter must be of type bool or point to a "
            "certificate file"
        )
        assert isinstance(
            warn_ssl, bool
        ), "'warn_ssl' parameter must be of type bool"

        self.username = username
        self.password = password
        self.api_host = api_host
        self.api_port = api_port
        self.use_ssl = use_ssl
        self.verify_ssl = verify_ssl
        self.warn_ssl = warn_ssl
        self.tenant_config = tenant

        if self.use_ssl:
            scheme = "https"
        else:
            scheme = "http"

        self.base_url = "{}://{}:{}".format(
            scheme, self.api_host, self.api_port
        )

        # Register endpoint modules - see @property definitions at end of file
        # for each module
        self._tenant = TenantController(self)
        self._config = ConfigController(self)
        self._gateway = GatewayController(self)
        self._k8s_worker = K8sWorkerController(self)
        self._k8s_cluster = K8sClusterController(self)
        self._license = LicenseController(self)
        self._lock = LockController(self)
        self._user = UserController(self)
        self._catalog = CatalogController(self)
        self._role = RoleController(self)
        self._datatap = DatatapController(self)

    def create_session(self):
        """Create a session with the HPE CP controller.

        Returns
        -------
        ContainerPlatformClient
            An instance of ContainerPlatformClient is returned.

        Raises
        ------
        APIException
            for connection error to the HPE CP controller
            requests.exceptions.RequestException
            for exceptions that are not a connection error
        """
        url = self.base_url + "/api/v1/login"
        auth = {"name": self.username, "password": self.password}

        if self.tenant_config:
            auth["tenant"] = self.tenant_config

        if self.warn_ssl is False:
            import urllib3

            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        # if self.log.level == 10:  # "DEBUG"
        #     if six.PY3:
        #         import http.client

        #         http.client.HTTPConnection.debuglevel = 1

        #     requests_log = logging.getLogger("requests.packages.urllib3")
        #     requests_log.setLevel(logging.DEBUG)
        #     requests_log.propagate = True

        response = None
        try:
            self.log.debug("REQ: {} : {} {}".format("Login", "post", url))
            response = requests.post(
                url, json=auth, verify=self.verify_ssl, timeout=10
            )  # 10 seconds
            response.raise_for_status()

        except (
            requests.exceptions.ConnectionError,
            NewConnectionError,
            MaxRetryError,
            ConnectTimeoutError,
        ) as e:
            self.log.debug(
                "RES: {} : {} {} {}".format("Login", "post", url, str(e))
            )
            if self.log.level == 10:  # "DEBUG"
                # The error is already output to the log, so this message
                # can be brief.
                msg = "Could not connect to the controller."
            else:
                msg = "Could not connect to the controller.\n" + str(e)
            raise_from(
                APIException(
                    message=msg,
                    request_method="post",
                    request_url=url,
                ),
                None,
            )

        except requests.exceptions.RequestException as e:
            if response is not None:
                self.log.error("Auth Response: " + response.text)
            else:
                self.log.error(e)
            raise

        # except Exception as e:
        #     if response is not None:
        #         self.log.error("Auth Response: " + response.text)
        #     else:
        #         self.log.error(e)
        #     raise_from(
        #         APIUnknownException(
        #             message=str(e), request_method="post", request_url=url,
        #         ),
        #         None,
        #     )

        self.session_headers = CaseInsensitiveDict(response.headers)
        self.session_id = CaseInsensitiveDict(response.headers)["location"]

        return self

    def _request_headers(self):

        headers = {
            "accept": "application/json",
            "X-BDS-SESSION": self.session_id,
            "cache-control": "no-cache",
            "content-type": "application/json",
        }
        return headers

    def _request(
        self,
        url,
        http_method="get",
        data={},
        description="",
        create_auth_headers=True,
        additional_headers={},
    ):
        """Make HTTP requests to the API host.

        Parameters
        ----------
        url : str
            This will be suffixed to the API host's address.
            Example '/api/v1/catalog/[0-9]+'
        http_method : str, optional
            HTTP method to be executed, by default "get"
        data : dict, optional
            Request payload (applicable for "post", "put"), by default {}
        description : str, optional
            Brief description about the request. , by default ""
        create_auth_headers : bool, optional
            By default True
        additional_headers : dict, optional
            Any additional headers to be passed while making the request,
            by default {}

        Returns
        -------
        Response
            The http response object

        Raises
        ------
        APIItemNotFoundException
        APIItemConflictException
        APIException
        """
        if create_auth_headers:
            headers = self._request_headers()
        else:
            headers = {}

        all_headers = {}
        all_headers.update(headers)
        all_headers.update(additional_headers)

        url = url = self.base_url + url

        if self.warn_ssl is False:
            import urllib3

            urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

        try:
            if http_method == "get":
                self.log.debug(
                    "REQ: {} : {} {}".format(description, http_method, url)
                )
                response = requests.get(
                    url, headers=all_headers, verify=self.verify_ssl
                )
            elif http_method == "put":
                self.log.debug(
                    "REQ: {} : {} {} {}".format(
                        description, http_method, url, json.dumps(data)
                    )
                )
                response = requests.put(
                    url,
                    headers=all_headers,
                    data=json.dumps(data),
                    verify=self.verify_ssl,
                )
            elif http_method == "post":
                self.log.debug(
                    "REQ: {} : {} {} {}".format(
                        description, http_method, url, json.dumps(data)
                    )
                )
                response = requests.post(
                    url,
                    headers=all_headers,
                    data=json.dumps(data),
                    verify=self.verify_ssl,
                )
            elif http_method == "delete":
                self.log.debug(
                    "REQ: {} : {} {}".format(description, http_method, url)
                )
                response = requests.delete(
                    url, headers=all_headers, verify=self.verify_ssl
                )

            response.raise_for_status()
        except requests.exceptions.RequestException as re:
            try:
                response_info = response.json()
            except Exception:
                response_info = response.text
            else:
                response_info = ""

            def log_response():
                self.log.debug(
                    "RES: {} : {} {} : {} {}".format(
                        description,
                        http_method,
                        url,
                        response.status_code,
                        response_info,
                    )
                )

            if response.status_code == 403:
                # This is expected for some method calls so do not log as an
                # error
                log_response()
                raise APIForbiddenException(
                    message=response_info,
                    request_method=http_method,
                    request_url=url,
                    request_data=json.dumps(data),
                )
            if response.status_code == 404:
                # This is expected for some method calls so do not log as an
                # error
                log_response()
                raise APIItemNotFoundException(
                    message=response_info,
                    request_method=http_method,
                    request_url=url,
                    request_data=json.dumps(data),
                )
            if response.status_code == 409:
                # This is expected for some method calls so do not log as an
                # error
                log_response()
                raise APIItemConflictException(
                    message=response_info,
                    request_method=http_method,
                    request_url=url,
                    request_data=json.dumps(data),
                )
            else:
                log_response()
                raise APIUnknownException(
                    message=str(re),  # get the exception message
                    request_method=http_method,
                    request_url=url,
                    request_data=json.dumps(data),
                )

        try:
            self.log.debug(
                "RES: {} : {} {} : {} {}".format(
                    description,
                    http_method,
                    url,
                    response.status_code,
                    json.dumps(response.json()),
                )
            )
        except ValueError:
            self.log.debug(
                "RES: {} : {} {} : {} {}".format(
                    description,
                    http_method,
                    url,
                    response.status_code,
                    response.text,
                )
            )

        return response

    @property
    def tenant(self):
        """Retrieve a reference to `.tenant.TenantController` object.

        See the class :py:class:`.tenant.TenantController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`list() <.tenant.TenantController.list>` in
        :py:class:`.tenant.TenantController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.tenant.list()
        """
        return self._tenant

    @property
    def config(self):
        """Retrieve a reference to `.config.ConfigController` object.

        See the class :py:class:`.config.ConfigController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`auth() <.config.ConfigController.auth>` in
        :py:class:`.config.ConfigController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.config.auth(
        ...        {
        ...            "external_identity_server":  {
        ...               "bind_pwd":"5ambaPwd@",
        ...                "user_attribute":"sAMAccountName",
        ...                "bind_type":"search_bind",
        ...                "bind_dn": (
        ...                     "cn=Administrator,CN=Users,DC=samdom,"
        ...                     "DC=example,DC=com"
        ...                     ),
        ...                "host":"10.1.0.77",
        ...                "security_protocol":"ldaps",
        ...                "base_dn":"CN=Users,DC=samdom,DC=example,DC=com",
        ...                "verify_peer": False,
        ...                "type":"Active Directory",
        ...                "port":636
        ...            }
        ...        }
        ...    )
        """  # noqa: E501
        return self._config

    @property
    def k8s_cluster(self):
        """Retrieve a reference to `.k8s_cluster.K8sClusterController` object.

        See the class :py:class:`.k8s_cluster.K8sClusterController` for the
        methods available.

        Example
        -------
        This example calls the method
        :py:meth:`list() <.k8s_cluster.K8sClusterController.list>` in
        :py:class:`.k8s_cluster.K8sClusterController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.k8s_cluster.list()
        """
        return self._k8s_cluster

    @property
    def k8s_worker(self):
        """Retrieve a reference to `.k8s_worker.K8sWorkerController` object.

        See the class :py:class:`.k8s_worker.K8sWorkerController` for the
        methods available.

        Example
        -------
        This example calls the method
        :py:meth:`list() <.k8s_worker.K8sWorkerController.list>` in
        :py:class:`.k8s_worker.K8sWorkerController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.k8s_worker.list()
        """
        return self._k8s_worker

    @property
    def gateway(self):
        """Retrieve a reference to `.gateway.GatewayController` object.

        See the class :py:class:`.gateway.GatewayController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`list() <.gateway.GatewayController.list>` in
        :py:class:`.gateway.GatewayController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.gateway.list()
        """
        return self._gateway

    @property
    def license(self):
        """Retrieve a reference to a `.license.LicenseController` object.

        See the class :py:class:`.license.LicenseController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`list() <.license.LicenseController.list>` in
        :py:class:`.license.LicenseController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.license.list()
        """
        return self._license

    @property
    def lock(self):
        """Retrieve a reference to a `.lock.LockController` object.

        See the class :py:class:`.lock.LockController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`get() <.lock.LockController.list>` in
        :py:class:`.lock.LockController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.lock.get()
        """
        return self._lock

    @property
    def log(self):
        """Retrieve a reference to a `:py:class:`.logger.Logger`.

        The log function can be
        called from controller objects via the `client` parameter passed in
        during instantiation of the controller.

        Example
        -------
        class K8sClusterController:
            ...

            def __init__(self, client):
                self.client = client

            def some_method(self):
                ...
                self.client.log.error("Some Error")
        """
        return self._log

    @property
    def user(self):
        """Retrieve a reference to a `.lock.LockController` object.

        See the class :py:class:`.lock.UserController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`get() <.user.UserController.get>` in
        :py:class:`.user.UserController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.user.get()
        """
        return self._user

    @property
    def catalog(self):
        """Retrieve a reference to a `.catalog.CatalogController` object.

        See the class :py:class:`.catalog.CatalogController` for the methods
        available.

        Example
        -------
        This example calls the method
        :py:meth:`create() <.catalog.CatalogController.create>` in
        :py:class:`.catalog.CatalogController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.catalog.create()
        """
        return self._catalog

    @property
    def role(self):
        """Retrieve a reference to a `.role.RoleController` object.

        See the class :py:class:`.role.RoleController` for the methods
        available.

        Example
        -------
        This example calls the method :py:meth:`get()
        <.role.RoleController.get>`
        in :py:class:`.role.RoleController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.role.get()
        """
        return self._role

    @property
    def datatap(self):
        """Retrieve a reference to a `.datatap.DatatapController` object.

        See the class :py:class:`.role.DatatapController` for the methods
        available.

        Example
        -------
        This example calls the method :py:meth:`get()
        <.datatap.DatatapController.get>`
        in :py:class:`.datatap.DatatapController`.

        >>> client = ContainerPlatformClient(...)
        >>> client.create_session()
        >>> client.datatap.get()
        """
        return self._datatap



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import abc
import json
import os
import sys
import jmespath
import six
import yaml
import wrapt
import traceback

from hpecp.logger import Logger

from hpecp import (
    APIException,
    APIItemConflictException,
    ContainerPlatformClient,
    ContainerPlatformClientException,
)

from hpecp.exceptions import (
    APIForbiddenException,
    APIItemNotFoundException,
    APIUnknownException,
)
from hpecp.cli_utils import TextOutput

_log = Logger.get_logger()


def get_profile():
    """Retrieve the profile - if supplied."""
    profile = os.getenv("PROFILE", default="default")
    _log.debug(
        "PROFILE envirionment variable exists with value '{}'".format(profile)
    )
    return profile


def get_config_file():
    """Retrieve the CLI config file."""
    if "HPECP_CONFIG_FILE" in os.environ:
        HPECP_CONFIG_FILE = os.path.expandvars(os.getenv("HPECP_CONFIG_FILE"))
        _log.debug(
            "HPECP_CONFIG_FILE env variable exists with value '{}'".format(
                HPECP_CONFIG_FILE
            )
        )
    else:
        HPECP_CONFIG_FILE = "~/.hpecp.conf"
        _log.debug(
            "HPECP_CONFIG_FILE env variable not found, setting to '{}'".format(
                HPECP_CONFIG_FILE
            )
        )
    return HPECP_CONFIG_FILE


@wrapt.decorator
def intercept_exception(wrapped, instance, args, kwargs):
    """Handle Exceptions."""  # noqa: D202

    def _unknown_exception_handler(ex):
        """Handle unknown exceptions."""
        if _log.level == 10:  # "DEBUG"
            print(
                "Unknown error.",
                file=sys.stderr,
            )
        else:
            print(
                "Unknown error. To debug run with env var LOG_LEVEL=DEBUG",
                file=sys.stderr,
            )
        tb = traceback.format_exc()
        _log.debug(tb)
        _log.debug(ex)
        sys.exit(1)

    try:
        return wrapped(*args, **kwargs)
    except SystemExit as se:
        sys.exit(se.code)
    except AssertionError as ae:
        print(ae, file=sys.stderr)
        sys.exit(1)
    except APIUnknownException as ue:
        _unknown_exception_handler(ue)
    except (
        APIException,
        APIItemNotFoundException,
        APIItemConflictException,
        APIForbiddenException,
        ContainerPlatformClientException,
    ) as e:
        print(e.message, file=sys.stderr)
        sys.exit(1)
    except Exception as ex:
        _unknown_exception_handler(ex)


@intercept_exception
def get_client(start_session=True):
    """Retrieve a reference to an authenticated client object."""
    client = ContainerPlatformClient.create_from_config_file(
        config_file=get_config_file(),
        profile=get_profile(),
    )
    if start_session:
        client.create_session()
    return client


@six.add_metaclass(abc.ABCMeta)
class BaseProxy:
    """Base 'proxy' class for generic calls to API."""

    def new_instance(self, client_module_name, resource_class):
        """Create a new instance (constructor).

        Parameters
        ----------
        client_module_name : str
            Name of the property in the ContainerPlatformClient that
            points to the different modules (user, gateway, cluster, etc)
        """
        self.client_module_name = client_module_name
        self.resource_class = resource_class
        super(BaseProxy, self).__init__()

    @intercept_exception
    def get(self, id, output="yaml", params=None):
        """Retrieve a Resource by ID.

        id: string
            the id of the resource with format: '/api/path/[0-9]+'
        output: string
            how to display the output, either 'yaml' or 'json', default 'yaml'
        """
        self.client = get_client()
        self.client_module_property = getattr(
            self.client, self.client_module_name
        )
        response = self.client_module_property.get(id=id, params=params)
        json_data = response.json

        if output == "json":
            print(json.dumps(json_data))
        elif output == "json-pp":
            print(
                json.dumps(
                    json_data,
                    indent=4,
                    sort_keys=True,
                )
            )
        else:

            print(
                yaml.dump(
                    yaml.load(
                        json.dumps(json_data),
                        Loader=yaml.FullLoader,
                    )
                )
            )

    @intercept_exception
    def delete(self, id, wait_for_delete_sec=0):
        """Delete a resource.

        :param id: the resource ID
        :param wait_for_delete_sec: wait for delete to complete
        (0 = do not wait)
        """
        self.client = get_client()
        self.client_module_property = getattr(
            self.client, self.client_module_name
        )
        self.client_module_property.delete(id=id)

        if wait_for_delete_sec > 0:
            self.wait_for_delete(id=id, timeout_secs=wait_for_delete_sec)

    @intercept_exception
    def list(self, output="table", columns="DEFAULT", query={}):
        """Retrieve the list of resources.

        Parameters
        ----------
        output : str, optional
            Define how the output should be printed, by default "table"
            "json" or "json-pp" (json pretty print) if providing a query
        columns : list/tuple, optional
            List of specific columns to be displayed, by default []
            `Catalog.default_display_fields`
            "DEFAULT", "WIDE", or columns list
        query : dict, optional
            Query in jmespath (https://jmespath.org/) format, by default {}
            if using a query, output must be "json" or "json-pp"
        """
        if columns is not None:
            if columns == "DEFAULT":
                columns = list(self.resource_class.default_display_fields)
            elif columns == "WIDE":
                columns = list(self.resource_class.all_fields)
            elif isinstance(columns, tuple):
                columns = list(columns)
            elif isinstance(columns, str):
                columns = columns.split(",")

        # FIXME: this also gets called by print_list()
        self.validate_list_params(
            all_fields=self.resource_class.all_fields,
            output=output,
            columns=columns,
            query=query,
        )

        self.client = get_client()
        self.client_module_property = getattr(
            self.client, self.client_module_name
        )
        list_instance = self.client_module_property.list()

        self.print_list(
            list_instance=list_instance,
            output=output,
            columns=columns,
            query=query,
        )

    @intercept_exception
    def validate_list_params(self, all_fields, output, columns, query):
        """Print a list of resources.

        Parameters
        ----------
        list_instance : [type]
            [description]
        output : [type]
            [description]
        columns : [type]
            [description]
        query : [type]
            [description]
        """
        if not isinstance(columns, list):
            print("'columns' parameter must be a list.", file=sys.stderr)
            sys.exit(1)
        for col in columns:
            if col not in all_fields:
                print("Unknown column '{}'.".format(col), file=sys.stderr)
                sys.exit(1)

        if len(query) == 0:
            if output not in ["table", "text"]:
                print(
                    "When providing the --columns param, the --output param "
                    "must be 'table' or 'text'",
                    file=sys.stderr,
                )
                sys.exit(1)
        else:
            if output not in ["json", "json-pp", "text"]:
                print(
                    (
                        "If you provide a jmes --query, the output must "
                        "be 'json', 'json-pp', or 'text'"
                    ),
                    file=sys.stderr,
                )
                sys.exit(1)

    @intercept_exception
    def print_list(self, list_instance, output, columns, query):
        """Print a list of resources.

        Parameters
        ----------
        list_instance : [type]
            [description]
        output : [type]
            [description]
        columns : [type]
            [description]
        query : [type]
            [description]
        """
        if columns == "DEFAULT":
            columns = list_instance.resource_class.default_display_fields
        elif columns == "WIDE":
            columns = list_instance.resource_class.all_fields

        self.validate_list_params(
            all_fields=list_instance.resource_class.all_fields,
            output=output,
            columns=columns,
            query=query,
        )

        # use tabulate for simplified user output
        if len(query) == 0:
            if output == "table":
                print(list_instance.tabulate(columns=columns))
            else:
                print(
                    list_instance.tabulate(
                        columns=columns, style="plain", display_headers=False
                    )
                )

        # user has provided a jmes query
        else:
            data = list_instance.json
            if output == "json-pp":
                print(
                    json.dumps(
                        jmespath.search(str(query), data),
                        indent=4,
                        sort_keys=True,
                    )
                )
            elif output == "text":
                obj = jmespath.search(str(query), data)
                print(TextOutput.dump(obj))
            else:
                print(
                    json.dumps(
                        jmespath.search(str(query), data),
                    )
                )

    def wait_for_state(
        self,
        id,
        states=[],
        timeout_secs=60,
    ):
        """See wait_for_status()."""
        self.wait_for_status(id, states, timeout_secs)

    def wait_for_status(
        self,
        id,
        status=[],
        timeout_secs=60,
    ):
        """Wait for resource to have one or more statuses.

        :param id: Resource id with format: /api/path/[0-9]+
        :param status: status(es) to wait for with format:
            ['status1', 'status2', 'statusn']
        :param timeout_secs: how many secs to wait before exiting
        :returns True/False if status was found within timeout_secs. May
            raise APIException.
        """
        self.client = get_client()
        self.client_module_property = getattr(
            self.client, self.client_module_name
        )

        resource_status = [
            self.client_module_property.status_class[s] for s in status
        ]

        _log.debug("resource_status = {}".format(resource_status))

        try:
            success = self.client_module_property.wait_for_status(
                id=id,
                status=resource_status,
                timeout_secs=timeout_secs,
            )
        except Exception:
            success = False

        if not success:
            print(
                "Failed to reach state(s) {} in {}s".format(
                    str(status),
                    str(timeout_secs),
                ),
                file=sys.stderr,
            )
            sys.exit(1)

    def wait_for_delete(
        self,
        id,
        timeout_secs=1200,
    ):
        """Wait for Gateway to be deleted.

        :param id: Cluster id with format: /api/v1/workers/[0-9]+
        :param timeout_secs: how many secs to wait before exiting
        :returns True if gateway was deleted within timeout_secs.
        """
        self.wait_for_state(
            id=id,
            timeout_secs=timeout_secs,
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from __future__ import absolute_import

from requests.structures import CaseInsensitiveDict

from .base_resource import AbstractResource, AbstractResourceController

try:
    basestring
except NameError:
    basestring = str


class User(AbstractResource):
    """Create an instance of User from json data returned from the HPE
    Container Platform API. Users of this library are not expected to create an
    instance of this class.

    Parameters
    ----------
    json : str
        The json returned by the API representing a User.

    Returns
    -------
    User
        An instance of User
    """

    all_fields = [
        "id",
        "name",
        "description",
        "is_group_added_user",
        "is_external",
        "is_service_account",
        "default_tenant",
        "is_siteadmin",
    ]
    # All of the fields of User objects as returned by the HPE Container
    # Platform API

    default_display_fields = [
        "id",
        "name",
        "description",
        "is_group_added_user",
        "is_external",
        "is_service_account",
        "default_tenant",
        "is_siteadmin",
    ]

    @property
    def id(self):
        """@Field: from json['_links']['self']['href'] -

        id format: '/api/v1/user/[0-9]+'
        """
        return self.json["_links"]["self"]["href"]

    @property
    def name(self):
        """@Field: from json['label']['name']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["label"]["name"]
            else:
                return self.json["label"]["name"]
        except Exception:
            return ""

    @property
    def description(self):
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["label"]["description"]
            else:
                return self.json["label"]["description"]
        except Exception:
            return ""

    @property
    def is_group_added_user(self):
        """@Field: from json['is_group_added_user']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["is_group_added_user"]
            else:
                return self.json["is_group_added_user"]
        except Exception:
            return ""

    @property
    def is_external(self):
        """@Field: from json['is_external']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["is_external"]
            else:
                return self.json["is_external"]
        except Exception:
            return ""

    @property
    def is_service_account(self):
        """@Field: from json['is_service_account']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["is_service_account"]
            else:
                return self.json["is_service_account"]
        except Exception:
            return ""

    @property
    def default_tenant(self):
        """@Field: from json['default_tenant']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["default_tenant"]
            else:
                return self.json["default_tenant"]
        except Exception:
            return ""

    @property
    def is_siteadmin(self):
        """@Field: from json['is_siteadmin']"""
        try:
            if "_embedded" in self.json:
                return self.json["_embedded"]["is_siteadmin"]
            else:
                return self.json["is_siteadmin"]
        except Exception:
            return ""

    @property
    def _links(self):
        """@Field: from json['_links']"""
        try:
            return self.json["_links"]
        except Exception:
            return ""


class UserController(AbstractResourceController):
    """Class that users will interact with to work with catalogs.

    An instance of this class is available in the
    client.ContainerPlatformClient with the attribute name
    :py:attr:`user <.client.ContainerPlatformClient.user>`.  The methods of
    this class can be invoked using `client.user.method()`.  See the example
    below:

    Example
    -------
    >>> client = ContainerPlatformClient(...).create_session()
    >>> client.user.list()
    """

    base_resource_path = "/api/v1/user/"
    resource_class = User
    resource_list_path = "users"

    def create(self, name, password=None, description="", is_external=True):
        """Create a user by specifying name and description.

        Parameters
        ----------
        name: str
            Client name.
        password: str
            Password
        description: str
            Description as a string.
        is_external: bool
            Set to True for external users

        Returns
        -------
        str
            The user ID
        """

        assert isinstance(
            name, basestring
        ), "'name' must be provided and must be a string"
        assert isinstance(
            description, basestring
        ), "'description must be a string"
        assert isinstance(
            is_external, bool
        ), "'is_external' must be provided and must be a bool"

        data = {
            "label": {"name": name, "description": description},
            "is_external": is_external,
        }

        if password:
            data["password"] = password

        response = self.client._request(
            url="/api/v1/user",
            http_method="post",
            data=data,
            description="user/create",
        )
        return CaseInsensitiveDict(response.headers)["location"]



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import sys
from hpecp.k8s_worker import WorkerK8sStatus, WorkerK8s
from hpecp.cli import base


class K8sWorkerProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.k8s_worker>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "create_with_ssh_key",
            "delete",
            "get",
            "list",
            "set_storage",
            "statuses",
            "wait_for_status",
        ]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(K8sWorkerProxy, self).new_instance("k8s_worker", WorkerK8s)

    @base.intercept_exception
    def create_with_ssh_key(
        self,
        ip=None,
        ssh_key=None,
        ssh_key_file=None,
        tags=[],
        ephemeral_disks=None,
        persistent_disks=None,
        wait_for_operation_secs=0,
    ):
        """Create a K8s Worker using SSH key authentication.

        Parameters
        ----------
        ip : str, optional
            The IP address of the host, this is used for internal
            communication, by default None.
        ssh_key : str, optional
            The SSH key data as a string, instead of this location to a key
            file may also be provided, by default None.
        ssh_key_file : str, optional
            The SSH key file path, by default None
        tags : list, optional
            Tags to use, e.g. "{ "tag1": "foo", "tag2": "bar"}", by default []
        ephemeral_disks : str
            Comma separated string containing ephemeral disks.
            e.g: "/dev/nvme2n1,/dev/nvme2n2"
        persistent_disks : str, optional
            Comma separated string containing persistent disks, by default
            None.
            e.g: "/dev/nvme1n1,/dev/nvme1n2"
        wait_for_operation_secs: int
            wait for operations to complete. 0 = don't wait
        """
        if ssh_key is None and ssh_key_file is None:
            print(
                "Either ssh_key or ssh_key_file must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if ssh_key is not None and ssh_key_file is not None:
            print(
                "Either ssh_key or ssh_key_file must be provided",
                file=sys.stderr,
            )
            sys.exit(1)

        if ssh_key_file:
            try:
                with open(ssh_key_file) as f:
                    ssh_key = f.read()
            except OSError:
                print(
                    "Could not open/read ssh-key-file: {}".format(
                        ssh_key_file
                    ),
                    file=sys.stderr,
                )
                sys.exit(1)

        if (
            ephemeral_disks is not None or persistent_disks is not None
        ) and wait_for_operation_secs == 0:
            print(
                (
                    "If setting disks, 'wait-for-operation-secs' parameter"
                    " must be greater than zero (recommended 600 seconds)"
                ),
                file=sys.stderr,
            )
            sys.exit(1)

        worker_id = base.get_client().k8s_worker.create_with_ssh_key(
            ip=ip,
            ssh_key_data=ssh_key,
            tags=tags,
        )

        if wait_for_operation_secs > 0:
            self.wait_for_status(
                id=worker_id,
                status=["storage_pending", "error"],
                timeout_secs=wait_for_operation_secs,
            )

        if base.get_client().k8s_worker.get(id=worker_id).status == "error":
            print(
                (
                    "Create request has errored. "
                    "Check status message with `hpecp k8sworker get {}".format(
                        id
                    )
                ),
                file=sys.stderr,
            )
            sys.exit(1)

        if ephemeral_disks is not None or persistent_disks is not None:
            self.set_storage(
                id=worker_id,
                ephemeral_disks=ephemeral_disks,
                persistent_disks=persistent_disks,
            )

        if wait_for_operation_secs > 0:
            self.wait_for_status(
                id=worker_id,
                status=["ready"],
                timeout_secs=wait_for_operation_secs,
            )

        print(worker_id)

    # TODO: verify with engineering if setup_log is a valid parameter
    # def get(self, id, setup_log=False):
    #     """Get a K8SWorker."""
    #     if setup_log is True:
    #         params = {"setup_log": "true"}
    #     else:
    #         params = {}
    #     return super(K8sWorkerProxy, self).get(id=id, params=params)

    @base.intercept_exception
    def set_storage(
        self,
        id,
        ephemeral_disks,
        persistent_disks=None,
    ):
        """Set storage for a k8s worker.

        Parameters
        ----------
        id : str
            The k8s worker ID
        ephemeral_disks : str
            Comma separated string containing ephemeral disks.
            e.g: "/dev/nvme2n1,/dev/nvme2n2"
        persistent_disks : str, optional
            Comma separated string containing persistent disks, by default
            None.
            e.g: "/dev/nvme1n1,/dev/nvme1n2"
        """
        if not ephemeral_disks:
            print("'ephemeral_disks' must be provided", file=sys.stderr)
            sys.exit(1)

        p_disks = (
            persistent_disks.split(",") if persistent_disks is not None else []
        )
        e_disks = ephemeral_disks.split(",")

        base.get_client().k8s_worker.set_storage(
            worker_id=id,
            persistent_disks=p_disks,
            ephemeral_disks=e_disks,
        )

    def statuses(
        self,
    ):
        """Return a list of valid statuses."""
        print([s.name for s in WorkerK8sStatus])



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

from textwrap import dedent
from hpecp.role import Role
from hpecp.cli import base


class RoleProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.role>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return ["delete", "examples", "get", "list"]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(RoleProxy, self).new_instance("role", Role)

    def examples(self):
        """Show examples for working with roles."""
        print(
            dedent(
                """\
                    
                # Retrieve the role ID for 'Admin'
                $ hpecp role list  --query "[?label.name == 'Admin'][_links.self.href] | [0][0]" --output json | tr -d '"'
                /api/v1/role/2
                """  # noqa:  E501
            )
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

from textwrap import dedent
from hpecp.tenant import Tenant
from hpecp.cli import base


class TenantProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.tenant>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "add_external_user_group",
            "assign_user_to_role",
            "create",
            "delete",
            "delete_external_user_group",
            "examples",
            "get",
            "get_external_user_groups",
            "k8skubeconfig",
            "list",
            # "status",  # TODO: implement me!
            "users",
            "wait_for_status",
        ]

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(TenantProxy, self).new_instance("tenant", Tenant)

    @base.intercept_exception
    def create(
        self,
        name=None,
        description=None,
        tenant_type=None,
        k8s_cluster_id=None,
    ):
        """Create a tenant.

        Parameters
        ----------
        name : [type], optional
            [description], by default None
        description : [type], optional
            [description], by default None
        tenant_type : [type], optional
            [description], by default None
        k8s_cluster_id : [type], optional
            [description], by default None
        """
        tenant_id = base.get_client().tenant.create(
            name=name,
            description=description,
            tenant_type=tenant_type,
            k8s_cluster_id=k8s_cluster_id,
        )
        print(tenant_id)

    def examples(self):
        """Show usage_examples of the list method."""
        print(
            dedent(
                """\

                hpecp tenant list --query "[?tenant_type == 'k8s']" --output json-pp

                """  # noqa: E501
            )
        )

    @base.intercept_exception
    def k8skubeconfig(self):
        """Retrieve the tenant kubeconfig.

        This requires the ContainerPlatformClient to be created with
        a 'tenant' parameter.

        Returns
        -------
        str
            Tenant KubeConfig
        """
        conf = base.get_client().tenant.k8skubeconfig()
        print(conf)

    @base.intercept_exception
    def users(self, id, output="table", columns="ALL", query={}):
        """Retrieve users assigned to tenant.

        Parameters
        ----------
        id : str
            The tenant ID.
        """
        list_instance = base.get_client().tenant.users(id=id)
        self.print_list(
            list_instance=list_instance,
            output=output,
            columns=columns,
            query=query,
        )

    @base.intercept_exception
    def assign_user_to_role(self, tenant_id, user_id, role_id):
        """Assign user to role in tenant."""
        base.get_client().tenant.assign_user_to_role(
            tenant_id=tenant_id, user_id=user_id, role_id=role_id
        )

    @base.intercept_exception
    def get_external_user_groups(self, tenant_id):
        """Retrieve External User Groups."""
        print(
            base.get_client().tenant.get_external_user_groups(
                tenant_id=tenant_id
            )
        )

    @base.intercept_exception
    def add_external_user_group(self, tenant_id, group, role_id):
        """Add External User Group."""
        base.get_client().tenant.add_external_user_group(
            tenant_id=tenant_id, group=group, role_id=role_id
        )

    @base.intercept_exception
    def delete_external_user_group(self, tenant_id, group):
        """Delete External User Group."""
        base.get_client().tenant.delete_external_user_group(
            tenant_id=tenant_id, group=group
        )



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

import json
import sys
import yaml

from hpecp.cli import base


class LockProxy(object):
    """Proxy object to :py:attr:`<hpecp.client.lock>`."""

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "create",
            "delete",
            "delete_all",
            "list",
        ]

    def list(
        self,
        output="yaml",
    ):
        """Get the system and user locks.

        :param output: how to display the output ['yaml'|'json']
        """
        if output not in ["yaml", "json"]:
            print(
                "'output' parameter must be 'yaml' or 'json'", file=sys.stderr
            )
            sys.exit(1)

        response = base.get_client().lock.get()

        if output == "yaml":
            print(
                yaml.dump(
                    yaml.load(
                        json.dumps(response),
                        Loader=yaml.FullLoader,
                    )
                )
            )
        else:
            print(json.dumps(response))

    @base.intercept_exception
    def create(
        self,
        reason,
    ):
        """Create a lock."""
        print(base.get_client().lock.create(reason), file=sys.stdout)

    @base.intercept_exception
    def delete(
        self,
        id,
    ):
        """Delete a user lock."""
        base.get_client().lock.delete(id)

    @base.intercept_exception
    def delete_all(
        self,
        timeout_secs=300,
    ):
        """Delete all locks."""
        success = base.get_client().lock.delete_all(timeout_secs=timeout_secs)
        if not success:
            print("Could not delete locks.", file=sys.stderr)
            sys.exit(1)



# (C) Copyright [2020] Hewlett Packard Enterprise Development LP
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""HPE Container Platform CLI."""

from __future__ import print_function

from textwrap import dedent
from hpecp.datatap import Datatap
from hpecp.cli import base


class DatatapProxy(base.BaseProxy):
    """Proxy object to :py:attr:`<hpecp.client.datatap>`."""

    def __init__(self):
        """Create instance of proxy class with the client module name."""
        super(DatatapProxy, self).new_instance("datatap", Datatap)

    def __dir__(self):
        """Return the CLI method names."""
        return [
            "create_hdfs_with_kerberos",
            "get",
            "list",
            "delete",
            # "examples",
            "wait_for_state",
        ]

    def examples(self):
        """Show examples for working with roles."""
        print(
            dedent(
                """\
                    Coming soon ...
                """  # noqa:  E501
            )
        )

    @base.intercept_exception
    def create_hdfs_with_kerberos(
        self,
        name,
        description="",
        path_from_endpoint="",
        kdc_data_host="",
        kdc_data_port="",
        realm="",
        client_principal="",
        browse_only=None,
        host="",
        keytab="",
        backup_host="",
        type="",
        port="",
        read_only=None,
    ):
        """TODO.

        Parameters
        ----------
        name : [type]
            [description]
        description : [type]
            [description]
        path_from_endpoint : [type]
            [description]
        kdc_data_host : [type]
            [description]
        kdc_data_port : [type]
            [description]
        realm : [type]
            [description]
        client_principal : [type]
            [description]
        browse_only : [type]
            [description]
        host : [type]
            [description]
        keytab : [type]
            [description]
        service_id : [type]
            [description]
        backup_host : [type]
            [description]
        type : [type]
            [description]
        port : [type]
            [description]
        read_only : [type]
            [description]
        """
        assert browse_only is not None and isinstance(
            browse_only, bool
        ), "'browse-only' parameter must be 'True' or 'False'"

        assert read_only is not None and isinstance(
            read_only, bool
        ), "'read-only' parameter must be 'True' or 'False'"

        base.get_client().datatap.create_hdfs_with_kerberos(
            name=name,
            description=description,
            path_from_endpoint=path_from_endpoint,
            kdc_data_host=kdc_data_host,
            kdc_data_port=kdc_data_port,
            realm=realm,
            client_principal=client_principal,
            browse_only=browse_only,
            host=host,
            keytab=keytab,
            backup_host=backup_host,
            type=type,
            port=port,
            read_only=read_only,
        )
